@@ -1,5 +1,5 @@
 package SimpleDB::Class::Cache;
-our $VERSION = '1.0102';
+our $VERSION = '1.0103';
 
 
 =head1 NAME
@@ -8,7 +8,7 @@ SimpleDB::Class::Cache - Memcached interface for SimpleDB.
 
 =head1 VERSION
 
-version 1.0102
+version 1.0103
 
 =head1 DESCRIPTION
 
@@ -95,6 +95,7 @@ Returns a L<Memcached::libmemcached> object, which is constructed using the info
 has 'memcached' => (
     is  => 'ro',
     lazy    => 1,
+    clearer => 'clear_memcached',
     default => sub {
         my $self = shift;
         my $memcached = Memcached::libmemcached::memcached_create();
@@ -154,7 +155,7 @@ The key to delete.
 
 sub delete {
     my $self = shift;
-    my ($domain, $id) = validate_pos(@_, { type => SCALAR }, { type => SCALAR });
+    my ($domain, $id, $retry) = validate_pos(@_, { type => SCALAR }, { type => SCALAR }, { optional => 1 } );
     my $key = $self->fix_key($domain, $id);
     my $memcached = $self->memcached;
     Memcached::libmemcached::memcached_delete($memcached, $key);
@@ -163,6 +164,18 @@ sub delete {
             error   => "Cannot connect to memcached server."
             );
     }
+    elsif ($memcached->errstr eq 'UNKNOWN READ FAILURE' ) {
+        if ($retry) {
+            SimpleDB::Class::Exception::Connection->throw(
+                error   => "Cannot connect to memcached server."
+            );
+        }
+        else {
+            warn "Memcached went away, reconnecting.";
+            $self->clear_memcached;
+            $self->delete($domain, $id, 1);
+        }
+    }
     elsif ($memcached->errstr eq 'NOT FOUND' ) {
        SimpleDB::Class::Exception::ObjectNotFound->throw(
             error   => "The cache key $key has no value.",
@@ -194,7 +207,7 @@ Throws SimpleDB::Class::Exception::Connection and SimpleDB::Class::Exception.
 =cut
 
 sub flush {
-    my ($self) = @_;
+    my ($self, $retry) = @_;
     my $memcached = $self->memcached;
     Memcached::libmemcached::memcached_flush($memcached);
     if ($memcached->errstr eq 'SYSTEM ERROR Unknown error: 0') {
@@ -202,6 +215,15 @@ sub flush {
             error   => "Cannot connect to memcached server."
         );
     }
+    elsif ($memcached->errstr eq 'UNKNOWN READ FAILURE' ) {
+        SimpleDB::Class::Exception::Connection->throw(
+            error   => "Cannot connect to memcached server."
+        ) if $retry;
+
+        warn "Memcached went away, reconnecting.";
+        $self->clear_memcached;
+        return $self->flush(1);
+    }
     elsif ($memcached->errstr eq 'NO SERVERS DEFINED') {
         SimpleDB::Class::Exception->throw(
             error   => "No memcached servers specified."
@@ -234,7 +256,7 @@ The key to retrieve.
 
 sub get {
     my $self = shift;
-    my ($domain, $id) = validate_pos(@_, { type => SCALAR }, { type => SCALAR });
+    my ($domain, $id, $retry) = validate_pos(@_, { type => SCALAR }, { type => SCALAR }, { optional => 1 });
     my $key = $self->fix_key($domain, $id);
     my $memcached = $self->memcached;
     my $content = Memcached::libmemcached::memcached_get($memcached, $key);
@@ -260,11 +282,16 @@ sub get {
             error   => "No memcached servers specified."
             );
     }
-    elsif ($memcached->errstr eq 'SYSTEM ERROR Unknown error: 0') {
+    elsif ($memcached->errstr eq 'SYSTEM ERROR Unknown error: 0' || $retry) {
         SimpleDB::Class::Exception::Connection->throw(
             error   => "Cannot connect to memcached server."
             );
     }
+    elsif ($memcached->errstr eq 'UNKNOWN READ FAILURE' ) {
+        warn "Memcached went away, reconnecting.";
+        $self->clear_memcached;
+        return $self->get($domain, $id, 1);
+    }
     SimpleDB::Class::Exception->throw(
         error   => "Couldn't get $key from cache because ".$memcached->errstr
     );
@@ -287,6 +314,7 @@ An array reference of domain names and ids to retrieve.
 sub mget {
     my $self = shift;
     my ($names) = validate_pos(@_, { type => ARRAYREF });
+    my $retry = shift;
     my @keys = map { $self->fix_key(@{$_}) } @{ $names };
     my %result;
     my $memcached = $self->memcached;
@@ -296,6 +324,14 @@ sub mget {
             error   => "Cannot connect to memcached server."
             );
     }
+    elsif ($memcached->errstr eq 'UNKNOWN READ FAILURE' ) {
+        SimpleDB::Class::Exception::Connection->throw(
+            error   => "Cannot connect to memcached server."
+            ) if $retry;
+        warn "Memcached went away, reconnecting.";
+        $self->clear_memcached;
+        return $self->get($names, 1);
+    }
     elsif ($memcached->errstr eq 'NO SERVERS DEFINED') {
         SimpleDB::Class::Exception->throw(
             error   => "No memcached servers specified."
@@ -345,7 +381,7 @@ A time in seconds for the cache to exist. Default is 3600 seconds (1 hour).
 
 sub set {
     my $self = shift;
-    my ($domain, $id, $value, $ttl) = validate_pos(@_, { type => SCALAR }, { type => SCALAR }, { type => HASHREF }, { type => SCALAR | UNDEF, optional => 1 });
+    my ($domain, $id, $value, $ttl, $retry) = validate_pos(@_, { type => SCALAR }, { type => SCALAR }, { type => HASHREF }, { type => SCALAR | UNDEF, optional => 1 }, { optional => 1 });
     my $key = $self->fix_key($domain, $id);
     $ttl ||= 60;
     my $frozenValue = Storable::nfreeze($value); 
@@ -354,11 +390,16 @@ sub set {
     if ($memcached->errstr eq 'SUCCESS') {
         return $value;
     }
-    elsif ($memcached->errstr eq 'SYSTEM ERROR Unknown error: 0') {
+    elsif ($memcached->errstr eq 'SYSTEM ERROR Unknown error: 0' || $retry) {
         SimpleDB::Class::Exception::Connection->throw(
             error   => "Cannot connect to memcached server."
             );
     }
+    elsif ($memcached->errstr eq 'UNKNOWN READ FAILURE' ) {
+        warn "Memcached went away, reconnecting.";
+        $self->clear_memcached;
+        return $self->set($domain, $id, $value, $ttl, 1);
+    }
     elsif ($memcached->errstr eq 'NO SERVERS DEFINED') {
         SimpleDB::Class::Exception->throw(
             error   => "No memcached servers specified."
